<?php

include THEME_DIR.'/admin/panel/settings.php';

/**
 * Theme Appearance Tab for Themify Custom Panel
 * @since 1.0.0
 * @return array
 */
function themify_theme_design_meta_box() {
    $state = themify_ternary_states(array(
	'icon_no' => THEMIFY_URI . '/img/ddbtn-check.svg',
	'icon_yes' => THEMIFY_URI . '/img/ddbtn-cross.svg',
    ));
    $opt = themify_ternary_options();
    $header_design_options = themify_theme_header_design_options();
    return array(
	// Notice
	array(
	    'name' => '_theme_appearance_notice',
	    'title' => '',
	    'description' => '',
	    'type' => 'separator',
	    'meta' => array(
		'html' => '<div class="themify-info-link">' . __('The settings here apply on this page only. Leave everything as default will use the site-wide Theme Appearance from the Themify panel > Settings > Theme Settings.', 'themify') . '</div>'
	    ),
	),
	// Header Design
	array(
	    'name' => 'header_design',
	    'title' => __('Header Design', 'themify'),
	    'description' => '',
	    'type' => 'layout',
	    'show_title' => true,
	    'meta' => $header_design_options,
	    'hide' => 'none header-leftpane header-minbar boxed-content header-rightpane',
	    'default' => 'default',
	),
	// Sticky Header
	array(
	    'name' => 'fixed_header',
	    'title' => __('Sticky Header', 'themify'),
	    'description' => '',
	    'type' => 'radio',
	    'meta' => $opt,
	    'class' => 'hide-if none header-leftpane header-minbar boxed-content header-rightpane',
	    'default' => 'default',
	),
	// Full Height Header
	array(
	    'name' => 'full_height_header',
	    'title' => __('Full Height Header', 'themify'),
	    'description' => __('Full height will display the container in 100% viewport height', 'themify'),
	    'type' => 'radio',
	    'meta' => $opt,
	    'class' => 'hide-if default none header-horizontal header-leftpane header-minbar boxed-content header-top-bar header-slide-out boxed-layout boxed-compact header-rightpane header-stripe header-menu-split header-overlay',
	    'default' => 'default',
	),
	// Header Elements
	array(
	    'name' => '_multi_header_elements',
	    'title' => __('Header Elements', 'themify'),
	    'description' => '',
	    'type' => 'multi',
	    'class' => 'hide-if none',
	    'meta' => array(
		'fields' => array(
		    // Show Site Logo
		    array(
			'name' => 'exclude_site_logo',
			'description' => '',
			'title' => __('Site Logo', 'themify'),
			'type' => 'dropdownbutton',
			'states' => $state,
			'class' => 'hide-if none header-menu-split',
			'after' => '<div class="clear"></div>',
		    ),
		    // Show Site Tagline
		    array(
			'name' => 'exclude_site_tagline',
			'description' => '',
			'title' => __('Site Tagline', 'themify'),
			'type' => 'dropdownbutton',
			'states' => $state,
			'class' => 'hide-if none',
			'after' => '<div class="clear"></div>',
		    ),
		    // Show Search Form
		    array(
			'name' => 'exclude_search_form',
			'description' => '',
			'title' => __('Search Form', 'themify'),
			'type' => 'dropdownbutton',
			'states' => $state,
			'class' => 'hide-if none',
			'after' => '<div class="clear"></div>',
		    ),
		    // Show RSS Link
		    array(
			'name' => 'exclude_rss',
			'description' => '',
			'title' => __('RSS Link', 'themify'),
			'type' => 'dropdownbutton',
			'states' => $state,
			'class' => 'hide-if none',
			'after' => '<div class="clear"></div>',
		    ),
		    // Show Header Widgets
		    array(
			'name' => 'exclude_header_widgets',
			'description' => '',
			'title' => __('Header Widgets', 'themify'),
			'type' => 'dropdownbutton',
			'states' => $state,
			'class' => 'hide-if none',
			'after' => '<div class="clear"></div>',
		    ),
		    // Show Social Widget
		    array(
			'name' => 'exclude_social_widget',
			'description' => '',
			'title' => __('Social Widget', 'themify'),
			'type' => 'dropdownbutton',
			'states' => $state,
			'class' => 'hide-if none',
			'after' => '<div class="clear"></div>',
		    ),
		    // Show Menu Navigation
		    array(
			'name' => 'exclude_menu_navigation',
			'description' => '',
			'title' => __('Menu Navigation', 'themify'),
			'type' => 'dropdownbutton',
			'states' => $state,
			'class' => 'hide-if none header-menu-split',
			'after' => '<div class="clear"></div>',
		    ),
		    array(
			'name' => 'exclude_cart_icon',
			'description' => '',
			'title' => __('Cart Icon', 'themify'),
			'type' => 'dropdownbutton',
			'states' => $state,
			'class' => '',
			'after' => '<div class="clear"></div>',
			'display_callback' => 'themify_is_woocommerce_active'
		    )
		),
		'description' => '',
		'before' => '',
		'after' => '<div class="clear"></div>',
		'separator' => ''
	    )
	),
	// Header Wrap
	array(
	    'name' => 'header_wrap',
	    'title' => __('Header Background Type', 'themify'),
	    'description' => '',
	    'type' => 'radio',
	    'show_title' => true,
	    'meta' => array(
		array(
		    'value' => 'solid',
		    'name' => __('Solid Color/Image', 'themify'),
		    'selected' => true
		),
		array(
		    'value' => 'transparent',
		    'name' => __('Transparent Header', 'themify'),
		),
		array(
		    'value' => 'slider',
		    'name' => __('Image Slider', 'themify'),
		),
		array(
		    'value' => 'video',
		    'name' => __('Video Background', 'themify'),
		),
		array(
		    'value' => 'colors',
		    'name' => __('Animating Colors', 'themify'),
		),
	    ),
	    'enable_toggle' => true,
	    'class' => 'hide-if none clear',
	    'default' => 'solid',
	),
	// Animated Colors
	array(
	    'name' => '_animated_colors',
	    'title' => __('Animating Colors', 'themify'),
	    'description' => sprintf(__('Animating Colors can be configured at <a href="%s">Themify > Settings > Theme Settings</a>', 'themify'), esc_url(add_query_arg('page', 'themify', admin_url('admin.php')))),
	    'type' => 'post_id_info',
	    'toggle' => 'colors-toggle',
	),
	// Select Background Gallery
	array(
	    'name' => 'background_gallery',
	    'title' => __('Header Slider', 'themify'),
	    'description' => '',
	    'type' => 'gallery_shortcode',
	    'toggle' => 'slider-toggle',
	    'class' => 'hide-if none',
	),
	array(
	    'type' => 'multi',
	    'name' => '_video_select',
	    'title' => __('Header Video', 'themify'),
	    'meta' => array(
		'fields' => array(
		    // Video File
		    array(
			'name' => 'video_file',
			'title' => __('Video File', 'themify'),
			'description' => '',
			'type' => 'video',
			'meta' => array(),
		    ),
		),
		'description' => __('Video format: mp4. Note: video background does not play on some mobile devices, background image will be used as fallback.', 'themify'),
		'before' => '',
		'after' => '',
		'separator' => ''
	    ),
	    'toggle' => 'video-toggle',
	    'class' => 'hide-if none',
	),
	// Background image
	array(
	    'name' => 'background_image',
	    'title' => '',
	    'type' => 'image',
	    'description' => '',
	    'meta' => array(),
	    'before' => '',
	    'after' => '',
	    'toggle' => array('solid-toggle', 'video-toggle'),
	    'class' => 'hide-if none',
	),
	// Background repeat
	array(
	    'name' => 'background_repeat',
	    'title' => '',
	    'description' => __('Background Image Mode', 'themify'),
	    'type' => 'dropdown',
	    'meta' => array(
		array(
		    'value' => '',
		    'name' => ''
		),
		array(
		    'value' => 'fullcover',
		    'name' => __('Fullcover', 'themify')
		),
		array(
		    'value' => 'repeat',
		    'name' => __('Repeat all', 'themify')
		),
		array(
		    'value' => 'no-repeat',
		    'name' => __('No repeat', 'themify')
		),
		array(
		    'value' => 'repeat-x',
		    'name' => __('Repeat horizontally', 'themify')
		),
		array(
		    'value' => 'repeat-y',
		    'name' => __('Repeat vertically', 'themify')
		),
	    ),
	    'toggle' => array('solid-toggle', 'video-toggle'),
	    'class' => 'hide-if none',
	),
	// Header Slider Auto
	array(
	    'name' => 'background_auto',
	    'title' => __('Autoplay', 'themify'),
	    'description' => '',
	    'type' => 'dropdown',
	    'meta' => array(
		array('value' => 'yes', 'name' => __('Yes', 'themify'), 'selected' => true),
		array('value' => 'no', 'name' => __('No', 'themify'))
	    ),
	    'toggle' => 'slider-toggle',
	    'default' => 'yes',
	),
	// Header Slider Auto Timeout
	array(
	    'name' => 'background_autotimeout',
	    'title' => __('Autoplay Timeout', 'themify'),
	    'description' => '',
	    'type' => 'dropdown',
	    'meta' => array(
		array('value' => 1, 'name' => __('1 Secs', 'themify')),
		array('value' => 2, 'name' => __('2 Secs', 'themify')),
		array('value' => 3, 'name' => __('3 Secs', 'themify')),
		array('value' => 4, 'name' => __('4 Secs', 'themify')),
		array('value' => 5, 'name' => __('5 Secs', 'themify'), 'selected' => true),
		array('value' => 6, 'name' => __('6 Secs', 'themify')),
		array('value' => 7, 'name' => __('7 Secs', 'themify')),
		array('value' => 8, 'name' => __('8 Secs', 'themify')),
		array('value' => 9, 'name' => __('9 Secs', 'themify')),
		array('value' => 10, 'name' => __('10 Secs', 'themify'))
	    ),
	    'toggle' => 'slider-toggle',
	    'default' => 5,
	),
	// Header Slider Transition Speed
	array(
	    'name' => 'background_speed',
	    'title' => __('Transition Speed', 'themify'),
	    'description' => '',
	    'type' => 'dropdown',
	    'meta' => array(
		array('value' => 1500, 'name' => __('Slow', 'themify')),
		array('value' => 500, 'name' => __('Normal', 'themify'), 'selected' => true),
		array('value' => 300, 'name' => __('Fast', 'themify'))
	    ),
	    'toggle' => 'slider-toggle',
	    'default' => 500,
	),
	// Header Slider Wrap
	array(
	    'name' => 'background_wrap',
	    'title' => __('Wrap Slides', 'themify'),
	    'description' => '',
	    'type' => 'dropdown',
	    'meta' => array(
		array('value' => 'yes', 'name' => __('Yes', 'themify'), 'selected' => true),
		array('value' => 'no', 'name' => __('No', 'themify'))
	    ),
	    'toggle' => 'slider-toggle',
	    'default' => 'yes',
	),
	// Hide Slider Controlls
	array(
	    'name' => 'header_hide_controlls',
	    'title' => __('Hide Slider Controlls', 'themify'),
	    'description' => '',
	    'type' => 'checkbox',
	    'toggle' => 'slider-toggle'
	),
    // Background Color
    array(
        'name' => 'background_color',
        'title' => __('Header Background', 'themify'),
        'description' => '',
        'type' => 'color',
        'meta' => array('default' => null),
        'toggle' => array('solid-toggle', 'slider-toggle', 'video-toggle'),
        'class' => 'hide-if none',
        'format' => 'rgba',
    ),
	// Header wrap text color
	array(
	    'name' => 'headerwrap_text_color',
	    'title' => __('Header Text Color', 'themify'),
	    'description' => '',
	    'type' => 'color',
	    'meta' => array('default' => null),
	    'class' => 'hide-if none',
	    'format' => 'rgba'
	),
	// Header wrap link color
	array(
	    'name' => 'headerwrap_link_color',
	    'title' => __('Header Link Color', 'themify'),
	    'description' => '',
	    'type' => 'color',
	    'meta' => array('default' => null),
	    'class' => 'hide-if none',
	    'format' => 'rgba'
	),
    );
}
function themify_admin_script_style() {
    wp_enqueue_style( 'themify-admin-css', themify_enque(THEME_URI . '/admin/css/admin.css'), null, Themify_Enqueue_Assets::$themeVersion);
    wp_enqueue_script('themify-admin-script', themify_enque(THEME_URI . '/admin/js/admin.js'), null, Themify_Enqueue_Assets::$themeVersion, true);
}
function themify_theme_setup_metaboxes($meta_boxes= array(), $post_type='all') {
    $supportedTypes=array('post', 'page', 'event', 'video', 'gallery', 'album', 'press');
    $dir=THEME_DIR . '/admin/pages/';
    if($post_type==='all'){
	add_action('themify_settings_panel_end', 'themify_admin_script_style');
	foreach($supportedTypes as $s){
	    require_once( $dir . "$s.php" );
	}
	return $meta_boxes;
    }
    if (!in_array($post_type, $supportedTypes, true)) {
	return $meta_boxes;
    }
    require_once( $dir . "$post_type.php" );
    $theme_metaboxes = call_user_func_array( "themify_theme_get_{$post_type}_metaboxes", array( array(), &$meta_boxes ) );

    return array_merge($theme_metaboxes, $meta_boxes);
}

if(isset( $_GET['page'] ) && $_GET['page']==='themify'){
    themify_theme_setup_metaboxes();
}
else{
    add_filter('themify_metabox/fields/themify-meta-boxes', 'themify_theme_setup_metaboxes', 10, 2);
}
