<?php

/**************************************************************************************************
 * Album Class - Shortcode
 **************************************************************************************************/

if ( ! class_exists( 'Themify_Album' ) ) {

	class Themify_Album  extends Themify_Shortcode_Base{

		protected $post_type = 'album';
		protected $tax = 'album-category';
		protected $tag = 'album-tag';
		protected $taxonomies;

		function __construct() {
			parent::__construct();
			if (!is_admin()){
				add_action('template_redirect', array($this,'lightbox'), 12);
			    add_filter('themify_query_posts_page_args',array(__CLASS__,'pre_get_post'));
			}
		}

		/**
		 * Register post type and taxonomy
		 */
		function register() {
			register_post_type( $this->post_type, array(
				'labels' => array(
					'name' => __('Albums', 'themify'),
					'singular_name' => __('Album', 'themify'),
					'add_new' => __( 'Add New', 'themify' ),
					'add_new_item' => __( 'Add New Album', 'themify' ),
					'edit_item' => __( 'Edit Album', 'themify' ),
					'new_item' => __( 'New Album', 'themify' ),
					'view_item' => __( 'View Album', 'themify' ),
					'search_items' => __( 'Search Albums', 'themify' ),
					'not_found' => __( 'No Albums found', 'themify' ),
					'not_found_in_trash' => __( 'No Albums found in Trash', 'themify' ),
					'menu_name' => __( 'Albums', 'themify' ),
				),
				'supports' => array('title', 'editor', 'thumbnail', 'custom-fields', 'excerpt', 'comments'),
				'hierarchical' => false,
				'public' => true,
				'query_var' => true,
				'can_export' => true,
				'capability_type' => 'post',
				'has_archive' => 'album-archive',
				'rewrite' => array(
					'slug' => themify_get( 'themify_album_slug',$this->post_type,true ),
				),
				'menu_icon' => 'dashicons-playlist-audio',
			));
			register_taxonomy( $this->tax, array( $this->post_type ), array(
				'labels' => array(
					'name' => __( 'Album Categories', 'themify' ),
					'singular_name' => __( 'Album Categories', 'themify' ),
				),
				'public' => true,
				'show_in_nav_menus' => true,
				'show_ui' => true,
				'show_tagcloud' => true,
				'hierarchical' => true,
				'rewrite' => array(
					'slug' => themify_get( 'themify_album_category_slug',$this->tax,true ),
				),
				'query_var' => true,
			));
			register_taxonomy( $this->tag, array( $this->post_type ), array(
				'labels' => array(
					'name' => __( 'Album Tags', 'themify' ),
					'singular_name' => __( 'Album Tags', 'themify' ),
				),
				'public' => true,
				'show_in_nav_menus' => true,
				'show_ui' => true,
				'show_tagcloud' => true,
				'hierarchical' => false,
				'rewrite' => array(
					'slug' => themify_get( 'themify_album_tag_slug',$this->tag,true ),
				),
				'query_var' => true,
			));
		}
		
		
		public static function pre_get_post($qpargs){
		    remove_filter('themify_query_posts_page_args',array(__CLASS__,'pre_get_post'));
		    global $themify;
		    if( 'album' === $themify->query_post_type && 'artist' === $themify->orderby){
			$qpargs['orderby'] = 'meta_value';
			$qpargs['meta_key'] = 'artist';
		    }
		    return $qpargs;
		}
		
		
		public function lightbox(){
			if (!empty( $_GET['post_in_lightbox'] )  && is_singular($this->post_type) ) {
			
				$return_template = locate_template( 'includes/single-'.$this->post_type.'.php' );
				include( $return_template );
				die();
			}
		}
	}
}

/**************************************************************************************************
 * Initialize Type Class
 **************************************************************************************************/
new Themify_Album();


/**
 * Callback for the [themify_playlist] shortcode
 */
function themify_music_playlist_shortcode( $atts = array(), $content = '' ) {

	    $atts['tracks']= strip_tags( nl2br( do_shortcode( $content ) ) );	
	    $atts['tracks']=json_decode('['.rtrim($atts['tracks'],',').']',true);
	    if(!isset($atts['images'])){
		    $atts['images']=false;
	    }
	    if(!isset($atts['tracknumbers'])){
		    $atts['tracknumbers']=false;
	    }
	    if(!isset($atts['artists'])){
		    $atts['artists']=false;
	    }
	    return wp_playlist_shortcode($atts);
}

/**
 * Callback for the [themify_trac] shortcode
 */
function themify_music_trac_shortcode( $atts = array(), $content = '' ) {
	if(empty($atts['src'])){
		return '';
	}
	$atts = shortcode_atts(
		array(
			'src'=>'',
			'type'=>  'audio/mpeg',
			'title' => '',
			'caption' => '',
			'thumb_src' => '',
		), $atts, 'themify_trac_shortcode' );

	$data['src']= esc_url( $atts['src'] );
	$data['type']= sanitize_text_field( $atts['type'] );
	if($atts['caption']!==''){
	    $data['caption']= sanitize_text_field( $atts['caption'] );
	}
	if( $atts['title']!==''){
	    $data['title']= sanitize_text_field( $atts['title'] );
	}
	if($atts['thumb_src']!==''){
	    $data['thumb']= array('src'=>esc_url( $atts['thumb_src'] ));
	    $size = themify_get_image_size($atts['thumb_src']);
	    if(isset($size['w'])) $data['thumb']['width']=$size['w'];
	    if(isset($size['h'])) $data['thumb']['height']=$size['h'];
	}
	return json_encode($data).',';

}

add_shortcode( 'themify_playlist', 'themify_music_playlist_shortcode');
add_shortcode( 'themify_trac','themify_music_trac_shortcode');