<?php
/* * ************************************************************************************************
 * Event Class - Shortcode
 * ************************************************************************************************ */

if (!class_exists('Themify_Event')) {

    class Themify_Event extends Themify_Shortcode_Base {

	protected $post_type = 'event';
	protected $tax = 'event-category';
	protected $tag = 'event-tag';
	protected $taxonomies;
	public static $date_time_format = 'Y-m-d H:i';

	function __construct() {
	    add_action('widgets_init', array($this, 'widget'));
	    parent::__construct();
	}

	/**
	 * Register post type and taxonomy
	 */
	function register() {
	    register_post_type($this->post_type, array(
		'labels' => array(
		    'name' => __('Events', 'themify'),
		    'singular_name' => __('Event', 'themify'),
		    'add_new' => __('Add New', 'themify'),
		    'add_new_item' => __('Add New Event', 'themify'),
		    'edit_item' => __('Edit Event', 'themify'),
		    'new_item' => __('New Event', 'themify'),
		    'view_item' => __('View Event', 'themify'),
		    'search_items' => __('Search Events', 'themify'),
		    'not_found' => __('No Events found', 'themify'),
		    'not_found_in_trash' => __('No Events found in Trash', 'themify'),
		    'menu_name' => __('Events', 'themify'),
		),
		'supports' => array('title', 'editor', 'author', 'thumbnail', 'custom-fields', 'excerpt', 'comments'),
		'hierarchical' => false,
		'public' => true,
		'query_var' => true,
		'can_export' => true,
		'capability_type' => 'post',
		'rewrite' => array(
		    'slug' => themify_get('themify_event_slug', $this->post_type, true),
		),
		'menu_icon' => 'dashicons-calendar',
	    ));
	    register_taxonomy($this->tax, array($this->post_type), array(
		'labels' => array(
		    'name' => __('Event Categories', 'themify'),
		    'singular_name' => __('Event Categories', 'themify'),
		),
		'public' => true,
		'show_in_nav_menus' => true,
		'show_ui' => true,
		'show_tagcloud' => true,
		'hierarchical' => true,
		'rewrite' => array(
		    'slug' => themify_get('themify_event_category_slug', $this->tax, true),
		),
		'query_var' => true
	    ));
	    register_taxonomy($this->tag, array($this->post_type), array(
		'labels' => array(
		    'name' => __('Event Tags', 'themify'),
		    'singular_name' => __('Event Tags', 'themify'),
		),
		'public' => true,
		'show_in_nav_menus' => true,
		'show_ui' => true,
		'show_tagcloud' => true,
		'hierarchical' => false,
		'rewrite' => array(
		    'slug' => themify_get('themify_event_tag_slug', $this->tag, true),
		),
		'query_var' => true,
	    ));
	    if (is_admin()) {
		add_action('admin_enqueue_scripts', array($this, 'meta_boxes_script'));
		add_filter("manage_edit-{$this->post_type}_columns", array($this, 'type_column_header'), 10, 2);
		add_action("manage_{$this->post_type}_posts_custom_column", array($this, 'type_column'), 10, 3);
	    }
	    else{
		add_filter('themify_query_posts_page_args',array(__CLASS__,'pre_get_post'));
	    }
	}

	/**
	 * Loads scripts in certain admin screens.
	 *
	 * @since 1.3.4
	 */
	function meta_boxes_script() {
	    $screen = get_current_screen();
	    if ($this->post_type === $screen->post_type) {
		wp_enqueue_script('themify-theme-custom-panel', themify_enque(THEME_URI . '/admin/js/themify-custom-panel.js'), null, Themify_Enqueue_Assets::$themeVersion, true);
		wp_localize_script('themify-theme-custom-panel', 'themifyMetaBox', array(
		    'validate' => array(
			'required' => '#start_date',
		    ),
		    'required_error' => esc_html__('This field is required.', 'themify'),
		));
	    }
	}

	/**
	 * Display an additional column in list
	 * @param array
	 * @return array
	 */
	function type_column_header($columns) {
	    unset($columns['date']);
	    $columns['icon'] = __('Image', 'themify');
	    $columns['start'] = __('Start Date', 'themify');
	    $columns['shortcode'] = __('Shortcode', 'themify');
	    return $columns;
	}

	/**
	 * Display shortcode, type, size and color in columns in tiles list
	 * @param string $column key
	 * @param number $post_id
	 * @return string
	 */
	function type_column($column, $post_id) {
	    switch ($column) {
		case 'shortcode':
		    echo '<code>[' . $this->post_type . ' id="' . $post_id . '"]</code>';
		    break;

		case 'start':
		    echo get_post_meta($post_id, 'start_date', true);
		    break;

		case 'icon':
		    $image = '';
		    if (has_post_thumbnail($post_id)) {
			$img = wp_get_attachment_image_src(get_post_thumbnail_id($post_id), 'thumbnail');
			if (isset($img[0])) {
			    $image = '<img src="' . $img[0] . '" width="50" height="50" />';
			}
		    }
		    if (!empty($image)) {
			echo $image;
		    } elseif ($video = get_post_meta($post_id, 'video_url', true)) {
			echo __('Has Featured Video', 'themify');
		    } else {
			echo __('No Featured Media', 'themify');
		    }
		    break;
	    }
	}

	/**
	 * Add shortcode to WP
	 * @param $atts Array shortcode attributes
	 * @return String
	 * @since 1.0.0
	 */
	function init_shortcode($atts) {
	    $attr = array(
		'id' => '',
		'title' => 'yes', // no
		'unlink_title' => 'no',
		'image' => 'yes', // no
		'unlink_image' => 'no',
		'post_meta' => 'yes', // no
		'post_date' => 'no', // yes
		'image_w' => '',
		'image_h' => '',
		'display' => 'none', // excerpt, none
		'more_link' => false, // true goes to post type archive, and admits custom link
		'more_text' => __('More &rarr;', 'themify'),
		'limit' => 3,
		'category' => 'all', // integer category ID
		'orderby' => 'event_date', // date, title, rand, event_date
		'order' => 'ASC', // DESC
		'style' => 'grid3', // grid4, grid2, list-post
		'use_original_dimensions' => 'no', // yes
		'auto' => 4, // autoplay pause length
		'effect' => 'scroll', // transition effect
		'speed' => 500, // transition speed
		'wrap' => 'yes',
		'slider_nav' => 'yes',
		'pager' => 'yes',
		// Shortcode specific
		'wrap_class' => '',
		'event_location' => 'yes',
		'event_date' => 'yes',
		'past_events' => ''
	    );
	    if (!isset($atts['style'])) {
		$atts['style'] = $attr['style'];
	    }
	    self::set_image_size($atts);
	    $shortcode_atts = shortcode_atts($attr, $atts);
	    return do_shortcode($this->shortcode($shortcode_atts));
	}

	/**
	 * Main shortcode rendering
	 * @param array $atts
	 * @param $post_type
	 * @return string|void
	 */
	function shortcode($atts = array()) {
	    $post_type= $this->post_type;
	    $args = array(
		'post_type' => $post_type,
		'posts_per_page' => $atts['limit'],
		'suppress_filters' => false,
		'no_found_rows' => true,
		'orderby' => $atts['orderby'],
		'order' => $atts['order'],
	    );
	    $args['tax_query'] = themify_parse_category_args($atts['category'], $this->tax );

	    if ($args['orderby'] === 'event_date') {
			$args['orderby'] = 'meta_value';
			$args['meta_key'] = 'start_date';
	    }
	    $multiple = true;

	    // Single post type or many single post types
	    if ('' != $atts['id']) {
		if (strpos($atts['id'], ',')) {
		    $ids = explode(',', str_replace(' ', '', $atts['id']));
		    foreach ($ids as $string_id) {
			$int_ids[] = (int) $string_id;
		    }
		    $args['post__in'] = $int_ids;
		    $args['orderby'] = 'post__in';
		} else {
		    $args['p'] = (int) $atts['id'];
		    $multiple = false;
		}
	    }
	    if ($atts['past_events'] === 'no') { // show only future events
            $args['meta_query'] = array(
                'relation' => 'OR',
                array(
                    'key' => 'end_date',
					'compare' => 'NOT EXISTS',
					'value' => 'null',
                ),
                array(
                    'key' => 'end_date',
                    'value' => date_i18n(self::$date_time_format),
                    'compare' => '>='
                ),
                array(
                    'key' => 'start_date',
                    'value' => date_i18n(self::$date_time_format),
                'compare' => '>='
                )
            );
	    }
	    $posts = get_posts(apply_filters('themify_' . $post_type . '_shortcode_args', $args));
	    $out = '';
	    if ($posts) {
		global $themify;
		$themify_save = clone $themify; // save a copy
		// override $themify object
		$themify->hide_title = 'yes' === $atts['title'] ? 'no' : 'yes';
		$themify->unlink_title = $atts['unlink_title'];
		$themify->hide_image = 'yes' === $atts['image'] ? 'no' : 'yes';
		$themify->unlink_image = $atts['unlink_image'];
		$themify->hide_meta = 'yes' === $atts['post_meta'] ? 'no' : 'yes';
		$themify->hide_date = 'yes' === $atts['post_date'] ? 'no' : 'yes';
		$themify->hide_meta_category = 'no';
		$themify->hide_meta_tag = 'no';
		$themify->hide_meta_author = 'yes';
		if (!$multiple) {
		    $themify->width = $atts['image_w'] !== '' ? $atts['image_w'] : get_post_meta($args['p'], 'image_width', true);
		    $themify->height = $atts['image_h'] !== '' ? $atts['image_h'] : get_post_meta($args['p'], 'image_height', true);
		} else {
		    $themify->width = $atts['image_w'];
		    $themify->height = $atts['image_h'];
		}
		$args = null;
		$themify->use_original_dimensions = 'yes' === $atts['use_original_dimensions'] ? 'yes' : 'no';
		$themify->display_content = $atts['display'];
		$themify->more_link = $atts['more_link'];
		$themify->more_text = $atts['more_text'];
		$themify->post_layout = explode(' ', $atts['style']);
		$themify->post_layout = trim($themify->post_layout[0]);
		$themify->media_position = themify_get('setting-default_media_position', 'above', true);
		$themify->hide_event_date = 'yes' === $atts['event_date'] ? 'no' : 'yes';
		$themify->hide_event_location = 'yes' === $atts['event_location'] ? 'no' : 'yes';
		$loop = themify_get_shortcode_template($posts, 'includes/loop', $post_type);
		if ($themify->post_layout==='slider') {
		    $loop = sprintf('<div class="slideshow-wrap"><div data-lazy="1" class="slideshow tf_carousel swiper-container tf_overflow tf_rel" data-auto="%s" data-effect="%s" data-speed="%s" data-wrapvar="%s" data-slider_nav="%s" data-pager="%s"><div class="swiper-wrapper tf_lazy tf_rel tf_w tf_h">',
				    is_numeric($atts['auto']) ? $atts['auto'] * 1000 : $atts['auto'],
				    $atts['effect'],
				    $atts['speed'],
				    $atts['wrap'],
				    $atts['slider_nav'],
				    $atts['pager']
			    ) . $loop . '</div></div></div>';
		}
		$cl=apply_filters( 'themify_loops_wrapper_class', array($atts['style'],$atts['wrap_class'],'shortcode',$post_type),$post_type,$themify->post_layout,'shortcode' );
		if(empty($themify->is_event_widget)){
		    $cl[]='loops-wrapper';
		}
		$cl=implode(' ',$cl);
		$out = "<div class='{$cl} tf_clear'>$loop</div>" . $this->section_link($atts['more_link'], $atts['more_text']);
		$themify = clone $themify_save; // revert to original $themify state
		$themify_save = null;
				wp_reset_postdata();
	    }
	    return $out;
	}

	/**
	 * Renders and returns the map for a given address.
	 * @param $address
	 * @return string
	 */
	public static function get_map($address) {
		ob_start();
		the_widget( 'Themify_Google_Maps', array(
			'address_map' => preg_replace("[\n|\r]", ' ', $address),
			'width' => '100%',
			'height' => 650,
			'scrollwheel_map' => 'disabled',
		) );
		$map = ob_get_clean();

		return $map;
	}

	/**
	 * Register widgets for this custom post type
	 */
	function widget() {
	    register_widget('Themify_Events');
	}


	
	public static function pre_get_post($qpargs){
	    remove_filter('themify_query_posts_page_args',array(__CLASS__,'pre_get_post'));
	    global $themify;
	    if( 'event' === $themify->query_post_type ){
		// ordered by event date 
		if ('meta_value' === $themify->orderby ) {
		    $qpargs['meta_key']='start_date';
		}
		if( 'yes' === themify_get( 'event_hide_past_events' ) ) { // show upcoming events only
			$qpargs['meta_query'] = array(
				'relation' => 'OR',
				array(
					'key' => 'end_date',
					'value' => date_i18n( self::$date_time_format ),
					'compare' => '>'
				),
				array(
					'key' => 'end_date',
					'compare' => 'NOT EXISTS'
				),
				array(
					'key' => 'end_date',
					'value' => '',
					'compare' => '='
				)
			);
		} 
		elseif( 'no' === themify_get( 'event_hide_past_events' ) ) { // show only past events
			$qpargs['meta_query'] = array(
				'relation' => 'AND',
				array(
					'key' => 'end_date',
					'value' => date_i18n( self::$date_time_format ),
					'compare' => '<'
				),
				array(
					'key' => 'end_date',
					'value' => '',
					'compare' => '!='
				)
			);
		}
	    }
	    return $qpargs;
	}

    }

}

/* * ************************************************************************************************
 * Initialize Type Class
 * ************************************************************************************************ */
new Themify_Event();

/* * ************************************************************************************************
 * Initialize Widget Class
 * ************************************************************************************************ */

class Themify_Events extends WP_Widget {

    function __construct() {
	/* Widget settings. */
	$widget_ops = array('classname' => 'event-posts', 'description' => __('A list of events.', 'themify'));

	/* Widget control settings. */
	$control_ops = array('id_base' => 'themify-event-posts');

	/* Create the widget. */
	parent::__construct('themify-event-posts', __('Themify - Event Posts', 'themify'), $widget_ops, $control_ops);
    }

    /**
     * Initialize widget
     *
     * @param array $args
     * @param array $instance
     */
    function widget($args, $instance) {
		/* User-selected settings. */
		global $themify;
		$title = apply_filters('widget_title', $instance['title']);
		$themify->is_event_widget = true;
		$attr = array(
			'image' => $instance['show_thumb'] ? 'yes' : 'no',
			'image_w' => $instance['thumb_width'],
			'image_h' => $instance['thumb_height'],
			'limit' => $instance['show_count'],
			'post_meta' => $instance['hide_meta'] ? 'no' : 'yes',
			'display' => 'none',
			'wrap_class'=>'event-widget',
			'style' => 'list-post',
			'category' => is_array($instance['category']) ? implode(',', $instance['category']) : $instance['category'],
			'event_location' => $instance['hide_event_location'] ? 'no' : 'yes',
			'event_date' => $instance['hide_event_date'] ? 'no' : 'yes',
			'title' => $instance['hide_title'] ? 'no' : 'yes',
			'past_events' => empty($instance['hide_past_events']) ? 'yes' : 'no'
		);
		if (isset($args['before_widget'])) {
			echo $args['before_widget'];
		}
		if ($title) {
			if (isset($args['before_title'])) {
				echo $args['before_title'];
			}
			echo $title;
			if (isset($args['after_title'])) {
				echo $args['after_title'];
			}
		}
		echo do_shortcode('[themify_event_posts ' . http_build_query($attr, '', ' ') . ']');
		
		if (isset($args['after_widget'])) {
			echo $args['after_widget'];
		}
		$themify->is_event_widget = false;
    }

    /**
     * Save widget options
     *
     * @param array $new_instance
     * @param array $old_instance
     *
     * @return array
     */
    function update($new_instance, $old_instance) {
	$instance = $old_instance;

	/* Strip tags (if needed) and update the widget settings. */
	$instance['title'] = strip_tags($new_instance['title']);
	$instance['category'] = $new_instance['category'];
	$instance['show_count'] = $new_instance['show_count'];
	$instance['show_thumb'] = $new_instance['show_thumb'];
	$instance['hide_title'] = $new_instance['hide_title'];
	$instance['thumb_width'] = $new_instance['thumb_width'];
	$instance['thumb_height'] = $new_instance['thumb_height'];

	$instance['hide_meta'] = $new_instance['hide_meta'];
	$instance['hide_event_location'] = $new_instance['hide_event_location'];
	$instance['hide_event_date'] = $new_instance['hide_event_date'];
	$instance['hide_past_events'] = $new_instance['hide_past_events'];

	return $instance;
    }

    /**
     * Render widget form
     * @param array $instance
     *
     * @return string|void
     */
    function form($instance) {

	/* Set up some default widget settings. */
	$defaults = array(
	    'title' => __('Events', 'themify'),
	    'category' => 0,
	    'show_count' => 5,
	    'show_thumb' => false,
	    'display' => 'none',
	    'hide_title' => false,
	    'thumb_width' => 313,
	    'thumb_height' => 200,
	    'hide_meta' => false,
	    'hide_event_location' => false,
	    'hide_event_date' => false,
	    'hide_past_events' => false,
	);
	$instance = wp_parse_args((array) $instance, $defaults);
	$categories = get_categories(array('taxonomy' => 'event-category'));
	?>

	<p>
	    <label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:', 'themify'); ?></label><br />
	    <input id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" value="<?php echo $instance['title']; ?>" width="100%" />
	</p>

	<p>
	    <label for="<?php echo $this->get_field_id('category'); ?>"><?php _e('Category:', 'themify');
	?></label>
	    <select id="<?php echo $this->get_field_id('category'); ?>" name="<?php echo $this->get_field_name('category'); ?>">
		<option value="0" <?php if (!$instance['category']) echo 'selected="selected"'; ?>><?php _e('All', 'themify'); ?></option>
		<?php
		foreach ($categories as $cat) {
		    echo '<option value="' . $cat->cat_ID . '"';
		    if ($cat->cat_ID == $instance['category'])
			echo ' selected="selected"';
		    echo '>' . $cat->cat_name . ' (' . $cat->category_count . ')';
		    echo '</option>';
		}
		?>
	    </select>
	</p>

	<p>
	    <input class="checkbox" type="checkbox" <?php checked($instance['hide_past_events'], 'on'); ?> id="<?php echo $this->get_field_id('hide_past_events'); ?>" name="<?php echo $this->get_field_name('hide_past_events'); ?>" />
	    <label for="<?php echo $this->get_field_id('hide_past_events'); ?>"><?php _e('Show upcoming events only', 'themify'); ?></label>
	<div class="description"><?php _e('If this option is not enabled, only past events are shown.', 'themify'); ?></div>
	</p>

	<p>
	    <label for="<?php echo $this->get_field_id('show_count'); ?>"><?php _e('Show:', 'themify'); ?></label>
	    <input id="<?php echo $this->get_field_id('show_count'); ?>" name="<?php echo $this->get_field_name('show_count'); ?>" value="<?php echo $instance['show_count']; ?>" size="2" /> <?php _e('events', 'themify'); ?>
	</p>

	<p>
	    <input class="checkbox" type="checkbox" <?php checked($instance['hide_title'], 'on'); ?> id="<?php echo $this->get_field_id('hide_title'); ?>" name="<?php echo $this->get_field_name('hide_title'); ?>" />
	    <label for="<?php echo $this->get_field_id('hide_title'); ?>"><?php _e('Hide event title', 'themify'); ?></label>
	</p>

	<p>
	    <input class="checkbox" type="checkbox" <?php checked($instance['show_thumb'], 'on'); ?> id="<?php echo $this->get_field_id('show_thumb'); ?>" name="<?php echo $this->get_field_name('show_thumb'); ?>" />
	    <label for="<?php echo $this->get_field_id('show_thumb'); ?>"><?php _e('Display featured image', 'themify'); ?></label>
	</p>

	<?php
	// only allow thumbnail dimensions if GD library supported
	if (function_exists('imagecreatetruecolor')) {
	    ?>
	    <p>
	        <label for="<?php echo $this->get_field_id('thumb_width'); ?>"><?php _e('Image size:', 'themify'); ?></label> <input type="text" id="<?php echo $this->get_field_id('thumb_width'); ?>" name="<?php echo $this->get_field_name('thumb_width'); ?>" value="<?php echo $instance['thumb_width']; ?>" size="3" /> x <input type="text" id="<?php echo $this->get_field_id('thumb_height'); ?>" name="<?php echo $this->get_field_name('thumb_height'); ?>" value="<?php echo $instance['thumb_height']; ?>" size="3" />
	    </p>
	    <?php
	}
	?>

	<p>
	    <input class="checkbox" type="checkbox" <?php checked($instance['hide_meta'], 'on'); ?> id="<?php echo $this->get_field_id('hide_meta'); ?>" name="<?php echo $this->get_field_name('hide_meta'); ?>" />
	    <label for="<?php echo $this->get_field_id('hide_meta'); ?>"><?php _e('Hide event meta', 'themify'); ?></label>
	</p>

	<p>
	    <input class="checkbox" type="checkbox" <?php checked($instance['hide_event_date'], 'on'); ?> id="<?php echo $this->get_field_id('hide_event_date'); ?>" name="<?php echo $this->get_field_name('hide_event_date'); ?>" />
	    <label for="<?php echo $this->get_field_id('hide_event_date'); ?>"><?php _e('Hide event date', 'themify'); ?></label>
	</p>

	<p>
	    <input class="checkbox" type="checkbox" <?php checked($instance['hide_event_location'], 'on'); ?> id="<?php echo $this->get_field_id('hide_event_location'); ?>" name="<?php echo $this->get_field_name('hide_event_location'); ?>" />
	    <label for="<?php echo $this->get_field_id('hide_event_location'); ?>"><?php _e('Hide event location', 'themify'); ?></label>
	</p>

	<?php
    }

}
