<?php
/***************************************************************************
 *  					Theme Functions
 * 	----------------------------------------------------------------------
 * 						DO NOT EDIT THIS FILE
 *	----------------------------------------------------------------------
 *
 *  					Copyright (C) Themify
 * 						https://themify.me
 *
 *  To add custom PHP functions to the theme, create a child theme (https://themify.me/docs/child-theme) and add it to the child theme functions.php file.
 *  They will be added to the theme automatically.
 *
 ***************************************************************************/

/////// Actions ////////
// Init post, page and additional post types if they exist
add_action( 'after_setup_theme', 'themify_theme_init_types' );

// Ajax Loading
add_action( 'init', 'themify_ajax_response');


// Set class on body tag according to layout width
add_filter( 'body_class', 'themify_theme_body_class', 99 );

// Exclude CPT for sidebar
add_filter( 'themify_exclude_CPT_for_sidebar', 'themify_CPT_exclude_sidebar' );

/**
 * Enqueue Stylesheets and Scripts
 * @since 1.0.0
 */
function themify_theme_enqueue_header() {

	$header = themify_theme_get_header_design();
	Themify_Enqueue_Assets::$disableGoogleFontsLoad=true;

	// Prepare JS variables
	$themify_script_vars = array(
		'ajax_nonce'	=> wp_create_nonce('ajax_nonce'),
		'is_ajax'=>!themify_check('setting-disable_ajax',true),
		'site_url'=>rtrim(home_url(),'/'),
		// Video File ratio
		'headerType'=>$header,
		// Auto play audio
		'autoplay' => themify_check( 'setting-audio_player_autoplay',true ) ? 'autoplay' : '',
	);
	if ( $themify_script_vars['is_ajax'] ) {
		$themify_script_vars['disable_link'] = array(
			esc_attr(get_bloginfo( 'rss2_url' )),
			esc_attr(get_bloginfo( 'comments_rss2_url' )),
			wp_login_url(),
			wp_logout_url(),
			admin_url(),
		);
	}
	wp_localize_script( 'themify-main-script', 'themifyScript', apply_filters('themify_script_vars', $themify_script_vars ) );
}

function themify_theme_enqueue_footer() {
	$header = themify_theme_get_header_design();
	$theme_version = Themify_Enqueue_Assets::$themeVersion;
	if ( $header !== 'default' && $header !== 'header-none' && $header!=='header-horizontal') {
		themify_enque_style( 'music-header', THEME_URI . '/styles/headers/' . $header . '.css', null, $theme_version );
		Themify_Enqueue_Assets::addMobileMenuCss('header-'.$header.'-menu',THEME_URI . '/styles/headers/media/'.$header.'.css');
		
	}
	if(!themify_check('setting-disable_ajax',true)){
		Themify_Enqueue_Assets::addPreLoadJs(THEME_URI . '/js/modules/themify.ajax.js',$theme_version);
	}
}

add_action( 'comment_form', 'themify_comment_form_css' );
function themify_comment_form_css(){
	remove_action( 'comment_form', 'themify_comment_form_css' );
	Themify_Enqueue_Assets::loadThemeStyleModule('comments');
}

/**
 * Load Google fonts used by the theme
 *
 * @return array
 */
function themify_theme_google_fonts( $fonts ) {
	/* translators: If there are characters in your language that are not supported by Lato, translate this to 'off'. Do not translate into your own language. */
	if ( 'off' !== _x( 'on', 'Lato font: on or off', 'themify' ) ) {
		$fonts['lato'] = 'Lato:100,300,400,700,900';
	}

	return $fonts;
}
add_filter( 'themify_google_fonts', 'themify_theme_google_fonts' );

/**
 * Checks if .animated-bg class must be output and sets the first animation color as initial background
 * Displays the data-bg and class attributes, including the background image and fullcover class
 *
 * @since 1.0.0
 *
 * @param string $area
 * @param string $classes Additional CSS classes to output for this element
 */
function themify_theme_header_background( $area = 'header', $classes = '' ) {
	$class = '';
	$style = '';
	$image = null;
	$repeat = '';
	$data_bg = '';
	$post_id = get_the_ID();
	$nobg = '';
	$area_wrap = themify_get( $area . '_wrap' );
	$image_meta = '';
	$background_gallery = '';

	if ( 'header' === $area && in_array( $area_wrap, array( 'video', 'slider' ) ) ) {
		$back_key = 'styling-background-header_wrap_background-background_image-value-value';
		$mode_key = 'styling-background-header_wrap_background-background_mode-value-value';
		$nobg_key = 'styling-background-header_wrap_background-background_image-value-none';
		$header_customizer = json_decode( get_theme_mod( 'headerwrap_background' ) );

		if ( isset( $header_customizer->src ) && '' != $header_customizer->src ) {
			$image = $header_customizer->src;
			$repeat = isset( $header_customizer->style ) ? $header_customizer->style : 'fullcover';
		} elseif ( themify_check( $back_key ) ) {
			$image = themify_get( $back_key );
			$repeat = themify_check( $mode_key )? themify_get( $mode_key ): 'fullcover';
		}
		$nobg = themify_get( $nobg_key ) || ( isset( $header_customizer->noimage ) && 'noimage' === $header_customizer->noimage );

		if ( is_singular( array( 'post', 'page', 'event', 'video', 'gallery', 'album', 'press' ) ) ) {
			global $themify;

			if ( is_page() ) {
				$post_id = $themify->page_id;
			} else {
				$post_id = get_the_ID();
			}

			$image_meta = get_post_meta($post_id, 'background_image', true);
			if ( $image_meta ) {
				$image = $image_meta;
				$repeat_meta = get_post_meta($post_id, 'background_repeat', true);
				if ( $repeat_meta ) {
					$repeat = $repeat_meta;
				}
			}
			$background_color = get_post_meta( $post_id, 'background_color', true );
			if ( $area_wrap == 'slider' && $background_color != '' ) {
				$data_bg .= "data-bgcolor='$background_color'";
			}
			$background_gallery = get_post_meta( $post_id, 'background_gallery', true );
			if ( $gallery = ( ( $area_wrap === 'slider' ) || ( $area_wrap == '' && $background_gallery != '' ) ) ) {
				$repeat = 'fullcover header-gallery';
			} elseif ( $nobg ) {
				// Checks $image_meta because $image might have been set in customizer and we don't need to check that.
				if ( is_singular() && $image_meta ) {
					$data_bg .= " data-bg='$image'";
				}
			} elseif ( $image ) {
				$data_bg .= " data-bg='$image'";
			} else {
				$data_bg .= ' data-bg="https://themify.me/demo/themes/wp-content/uploads/2013/05/header.jpg"';
				$repeat = 'fullcover';
			}

			$video = get_post_meta( $post_id, 'video_file', true );

			if ( 'video' === $area_wrap && $video != '' ) {
				$data_bg .= " data-fullwidthvideo='$video'";
			}
		}
	}

	if ( $repeat || $classes ) {
		$class = "class='$repeat $classes'";
	}

	if ( !empty( $style ) ) {
		$style = 'style="' . $style . '"';
	}
	
	echo "$data_bg $class $style";
}

/**
 * Returns options for header design. Used in custom panel and theme settings.
 *
 * @since 1.0.0
 *
 * @return array List of header design options.
 */
function themify_theme_header_design_options() {
	return apply_filters( 'themify_theme_header_design_options', array(
		array(
			'value' => 'default',
			'img'   => 'themify/img/default.svg',
			'title' => __( 'Default', 'themify' ),
			'selected' => true,
		),
		array(
			'value' => 'header-block',
			'img'   => 'images/layout-icons/header-block.png',
			'title' => __( 'Header Block', 'themify' ),
		),
		array(
			'value' => 'header-horizontal',
			'img'   => 'images/layout-icons/header-horizontal.png',
			'title' => __( 'Header Horizontal', 'themify' ),
		),
		array(
			'value' => 'header-boxed-layout',
			'img'   => 'images/layout-icons/header-boxed-layout.png',
			'title' => __( 'Header Boxed Layout', 'themify' ),
		),
		array(
			'value' => 'header-leftpane', // hides sticky header
			'img'   => 'images/layout-icons/header-leftpanel.png',
			'title' => __( 'Header Left Pane', 'themify' ),
		),
		array(
			'value' => 'header-slide-out', // hides sticky header
			'img'   => 'images/layout-icons/header-slide-out.png',
			'title' => __( 'Header Slide Out', 'themify' ),
		),
		array(
			'value' => 'header-minbar', // hides sticky header
			'img'   => 'images/layout-icons/header-minbar.png',
			'title' => __( 'Header Min Bar', 'themify' ),
		),
		array(
			'value' => 'header-boxed-compact', // hides sticky header
			'img'   => 'images/layout-icons/header-boxed-compact.png',
			'title' => __( 'Header Boxed Compact', 'themify' ),
		),
		array(
			'value' => 'header-overlay', // hides sticky header
			'img'   => 'images/layout-icons/header-overlay.png',
			'title' => __( 'Header Overlay ', 'themify' ),
		),
		array(
			'value' => 'header-rightpane', // hides sticky header
			'img'   => 'images/layout-icons/header-rightpanel.png',
			'title' => __( 'Header Right Pane ', 'themify' ),
		),
		array(
			'value' => 'none', // hides sticky header
			'img'   => 'images/layout-icons/none.png',
			'title' => __( 'No Header ', 'themify' ),
		)
	));
}

function themify_get_header_design_value(){
	static $is = null;
	if($is===null){
		$is = wp_list_pluck( themify_theme_header_design_options(), 'value' );
	}
	return $is;
}

function themify_theme_get_header_design() {
	static $is = null;
	if($is===null){
		$is = themify_area_design( 'header', array(
			'default' => '',
			'values' => themify_get_header_design_value(),
		) );
		if($is==='none'){
			$is = 'header-none';
		}
	}
	return $is;
}

if ( ! function_exists( 'themify_theme_body_class' ) ) {
	/**
	 * Adds body classes for special theme features.
	 *
	 * @since 1.0.0
	 *
	 * @param $classes
	 *
	 * @return array
	 */
	function themify_theme_body_class( $classes ) {
		
		$header = themify_theme_get_header_design();
		$classes[] = $header;
		$fixedHeader=themify_theme_fixed_header();
		if($fixedHeader===''){
		    $fixedHeader='no-fixed-header';
		}
		$classes[] = $fixedHeader;
		if ( 'transparent' == themify_get( 'header_wrap' ) && ( is_single() || is_page() || themify_is_shop() )) {
			Themify_Enqueue_Assets::loadThemeStyleModule('transparent-header');
			$classes[] = 'transparent-header';
		}
		if ( 'header-block' === $header && 'yes' === themify_get_both( 'full_height_header','setting-full_height_header' ) ) {
			Themify_Enqueue_Assets::loadThemeStyleModule('header-full-height');
			$classes[] = 'header-full-height';
		}
		
		// Exclusion classes
		if ( ! themify_theme_show_area( 'site_logo' ) ) {
			$classes[] = 'logo-off';
		}
		if ( ! themify_theme_show_area( 'site_tagline' ) ) {
			$classes[] = 'tagline-off';
		}
		if ( ! themify_theme_show_area( 'social_widget' ) ) {
			$classes[] = 'social-widget-off';
		}
		if ( ! themify_theme_show_area( 'rss' ) ) {
			$classes[] = 'rss-off';
		}
		if ( ! themify_theme_show_area( 'search_form' ) ) {
			$classes[] = 'search-off';
		}
		if ( ! themify_theme_show_area( 'menu_navigation' ) ) {
			$classes[] = 'menu-navigation-off';
		}
		if ( ! themify_theme_show_area( 'header_widgets' ) ) {
			$classes[] = 'header-widgets-off';
		}
		if ( is_singular()){
			$post_type = get_post_type();
			if($post_type==='event' || $post_type==='album' || $post_type==='gallery' || $post_type==='post'){
				if($post_type==='gallery'){
					Themify_Enqueue_Assets::loadGalleryCss();
				}
				Themify_Enqueue_Assets::loadThemeStyleModule( 'single/' . $post_type);
			}
		}
		return $classes;
	}
}

if ( ! function_exists( 'themify_theme_show_area' ) ) {
	/**
	 * Checks whether to show a certain layout area or not.
	 *
	 * @param string $area
	 *
	 * @return bool
	 */
	function themify_theme_show_area( $area = '' ) {
		$show = true;
		switch ( $area ) {
			case 'header':
				$show = 'none' === themify_area_design( $area, array( 'values'  => wp_list_pluck( themify_theme_header_design_options(), 'value' ) ) ) ? false : true;
				break;
			default:
				if ( ( is_singular() && ! is_attachment() ) || themify_is_shop() ) {
					$exclude = themify_get( 'exclude_' . $area );
					if ( 'yes' === $exclude ) {
						$show = false;
					} elseif ( 'no' === $exclude ) {
						$show = true;
					} else {
						$show = themify_check( 'setting-exclude_' . $area,true ) ? false : true;
					}
				} elseif ( themify_check( 'setting-exclude_' . $area,true ) ) {
					$show = false;
				}
				break;
		}

		return apply_filters( "themify_theme_show_{$area}", $show );
	}
}

if ( ! function_exists( 'themify_theme_do_not_exclude_all' ) ) {
	/**
	 * Checks if all the elements in a certain area have been excluded.
	 *
	 * @since 1.0.0
	 *
	 * @param string $area
	 *
	 * @return bool
	 */
	function themify_theme_do_not_exclude_all( $area = 'header' ) {
		if ( 'mobile-menu' === $area ) {
			return themify_theme_show_area( 'search_form' ) || themify_theme_show_area( 'social_widget' ) || themify_theme_show_area( 'header_widgets' ) || themify_theme_show_area( 'rss' ) || themify_theme_show_area( 'menu_navigation' );
		}
		elseif ( 'header' === $area ) {
			return themify_theme_show_area( 'site_logo' ) || themify_theme_show_area( 'site_tagline' ) || themify_theme_show_area( 'search_form' ) || themify_theme_show_area( 'social_widget' ) || themify_theme_show_area( 'header_widgets' ) || themify_theme_show_area( 'rss' ) || themify_theme_show_area( 'menu_navigation' );
		}
		return false;
	}
}


if ( ! function_exists( 'themify_theme_custom_post_css' ) ) {
	/**
	 * Outputs custom post CSS at the end of a post
	 * @since 1.0.0
	 */
	function themify_theme_custom_post_css() {
		global $themify;
		if (is_singular() && in_array( get_post_type(), array( 'event', 'gallery', 'video', 'album', 'press', 'post', 'page') ) ) {
			$post_id = get_the_ID();
			if ( is_page() ) {
				if ( is_post_type_archive( 'product' ) ) {
					$entry_id = '.post-type-archive-product';
				} else {
					$entry_id = '.page-id-' . $post_id;
				}
			} else {
				$entry_id = '.postid-' . $post_id;
			}
			$headerwrap = $entry_id . ' #headerwrap';
			$site_logo = $entry_id . ' #site-logo';
			$site_description = $entry_id . ' #site-description';
			$main_nav = $entry_id . ' #main-nav';
			$social_widget = $entry_id . ' .sidemenu .social-widget';
			$css = array();
			$style = '';
			$rules = array(
				$headerwrap => array(),
				"$entry_id #site-logo span:after, $entry_id #headerwrap #searchform, $entry_id #main-nav .current_page_item a, $entry_id #main-nav .current-menu-item a" => array(
					array(
							'prop' => 'border-color',
							'key'  => 'headerwrap_text_color'
					),
				),
			);
			$header_type = themify_get( 'header_wrap' );
			if ( 'transparent' !== $header_type ) {
				$rules = array(
					"$entry_id #site-logo span:after, $entry_id #headerwrap #searchform, $entry_id #main-nav .current_page_item a, $entry_id #main-nav .current-menu-item a" => array(
							array(
								'prop' => 'border-color',
								'key'  => 'headerwrap_text_color'
							),
					),
				);
				if ( 'solid' === $header_type || !$header_type || 'video' === $header_type ) {
					$rules[$headerwrap] = array(
						array(
							'prop' => 'background-color',
							'key'  => 'background_color'
						),
						array(
							'prop' => 'background-image',
							'key'  => 'background_image',
							'dependson' => array(
								'prop' => 'background-repeat',
								'key'  => 'background_repeat'
							),
						),
					);
				}
			}

			$rules["$headerwrap, $site_logo, $site_description"] = array(
				array(
					'prop' => 'color',
					'key'  => 'headerwrap_text_color'
				),
			);

			$rules["$site_logo a, $site_description a, $social_widget a, $main_nav > li > a"] = array(
				array(
					'prop' => 'color',
					'key'  => 'headerwrap_link_color'
				),
			);

			foreach ( $rules as $selector => $property ) {
				foreach ( $property as $val ) {
					$prop = $val['prop'];
					$key = $val['key'];
					if ( is_array( $key ) ) {
						if ( $prop == 'font-size' && themify_check( $key[0] ) ) {
							$css[$selector][$prop] = $prop . ': ' . themify_get( $key[0] ) . themify_get( $key[1] );
						}
					} elseif ( themify_check( $key ) && 'default' !== ($v=themify_get( $key )) ) {
						if ( $prop === 'color' || stripos( $prop, 'color' ) ) {
							if ( 'scheme_background' == $key ) {
								$header_bg_opacity = 'transparent' === themify_get( 'header_wrap' ) ? '0.9': '1';
								$css[$selector][$prop] = $prop . ':' . 'rgba(' . themify_theme_hex2rgb( themify_get( $key ) ) . ', ' . $header_bg_opacity . ')';
							} else {
								$css[$selector][$prop] = $prop . ': ' . themify_get_color( $key );
							}
						}
						elseif ( $prop === 'background-image' && 'default' !== $v ) {
							if(isset($val['dependson']) &&  $val['dependson']['prop']==='background-repeat'){
								$v2=themify_get(  $val['dependson']['key'] );
								if($v2==='fullcover' && $selector==='.skin-styles'){
									$selector.=':before';
								}
							}
							themify_generateWebp($v);
							$css[$selector][$prop] = $prop .': url(' . $v . ')';
							if(isset($v2)){
								$css[$selector]['background-size'] = 'background-size:cover;';
								if($v2==='fullcover' &&  $selector==='.skin-styles:before'){
									$css[$selector]['background-size'].='content:"";position:fixed;width:100%;height:100%;left:0;top:0;pointer-events:none;z-index:-1';
								}
								else{
									$css[$selector]['background-size'].= 'background-attachment:fixed';
								}
								$css[$selector][$prop] = $prop .': url(' . $v . ')';
							}
						}
						elseif ( $prop === 'font-family' ) {
							$font = themify_get( $key );
							$css[$selector][$prop] = $prop .': '. $font;
							if ( ! in_array( $font, themify_get_web_safe_font_list( true ) ) ) {
								$themify->google_fonts .= str_replace( ' ', '+', $font.'|' );
							}
						}
						else {
							$css[$selector][$prop] = $prop .': '. themify_get( $key );
						}
					}
				}
				if ( ! empty( $css[$selector] ) ) {
					$style .= "$selector {\n\t" . implode( ";\n\t", $css[$selector] ) . "\n}\n";
				}
			}

			if ( '' != $style ) {
				echo "\n<!-- Entry Style -->\n<style id=\"tf_theme_entry_css\">\n$style</style>\n<!-- End Entry Style -->\n";
			}
		}
	}
	add_action( 'wp_head', 'themify_theme_custom_post_css', 77 );

}


/* Custom Write Panels
/***************************************************************************/

///////////////////////////////////////
// Build Write Panels
///////////////////////////////////////

if ( ! function_exists( 'themify_theme_init_types' ) ) {
	/**
	 * Initialize custom panel with its definitions
	 * Custom panel definitions are located in admin/post-type-TYPE.php
	 * @since 1.0.0
	 */
	function themify_theme_init_types() {
		add_theme_support( 'post-thumbnails' );
		// Load Themify Social Share
		require_once THEME_DIR.'/class-social-share.php';

		// Load required files for post, page and custom post types where it applies
		foreach ( array('base','event', 'gallery', 'video', 'album', 'press' ) as $type ) {
			require_once( THEME_DIR."/admin/post-type-$type.php" );
		}
		register_nav_menus( array(
			'main-nav' => __( 'Main Navigation', 'themify' ),
			'footer-nav' => __( 'Footer Navigation', 'themify' ),
		));
		if(is_child_theme()){
		    include THEME_DIR.'/deprecated.php';
		}
	}
}

if( ! function_exists('themify_CPT_exclude_sidebar') ) {
	/**
	 * Exclude Custom Post Types
	 */
	function themify_CPT_exclude_sidebar($CPT = array()) {
		$CPT[] = 'video';
		$CPT[] = 'event';
		$CPT[] = 'gallery';
		$CPT[] = 'album';
		$CPT[] = 'press';
		return $CPT;
	}
}


if ( ! function_exists( 'themify_theme_likeit' ) ) {
	/**
	 * Checks liker's IP and saves it to the post if it's not already in likers list.
	 * @since 1.0.0
	 */
	function themify_theme_likeit() {
		check_ajax_referer( 'ajax_nonce', 'nonce' );

		$post_id = $_POST['post_id'];

		$ip = $_SERVER['REMOTE_ADDR'];

		$current_likers = trim( get_post_meta($post_id, 'likers', true) );

		if( isset( $current_likers ) && '' != $current_likers ) {
			$current_likers_count = count( explode( ',', $current_likers ) );
		} else {
			$current_likers_count = 0;
		}

		if( false === stripos( $current_likers, $ip ) ) {

			$save_likers = isset( $current_likers ) && '' != $current_likers? $current_likers . ',' . $ip :	$ip;

			$update_result = update_post_meta($post_id, 'likers', $save_likers);

			if ( $update_result ) {
				echo json_encode( array(
					'status' => 'new',
					'likers' => $current_likers_count + 1,
					'ip' => $ip
				) );
			} else {
				echo json_encode( array(
					'status' => 'failed',
					'ip' => $ip
				) );
			}
		} else {
			echo json_encode( array(
				'status' => 'isliker',
				'ip' => $ip
			) );
		}
		die();
	}
	add_action('wp_ajax_themify_likeit', 'themify_theme_likeit');
	add_action('wp_ajax_nopriv_themify_likeit', 'themify_theme_likeit');
}

if ( ! function_exists( 'themify_theme_get_like' ) ) {
	/**
	 * Return number of likers or 0
	 * @param bool $echo Whether to echo or just return
	 * @return string
	 * @since 1.0.0
	 */
	function themify_theme_get_like( $echo = false ) {
		if( $current_likers = themify_get('likers') ) {
			$likers = explode(',', trim($current_likers));
			$count = count($likers);
		} else {
			$count = '0';
		}
		if ( $echo ) echo $count;
		return $count;
	}
}

if ( ! function_exists( 'themify_theme_is_liker' ) ) {
	/**
	 * Check if visitor already likes this entry
	 * @param bool $echo Whether to echo or just return
	 * @return string
	 * @since 1.0.0
	 */
	function themify_theme_is_liker( $post_id ) {
		$current_likers = trim( get_post_meta($post_id, 'likers', true) );
		if( false === stripos( $current_likers, $_SERVER['REMOTE_ADDR'] ) ) {
			// IP wasn't found among likers, he doesn't like this yet
			return false;
		} else {
			// IP was found among likers, he likes this already
			return true;
		}
	}
}

if ( ! function_exists( 'themify_theme_featured_area_style' ) ) {
	function themify_theme_featured_area_style() {
		return themify_get('background_repeat','');
	}
}


/**
 * Allow featured areas to launch fullscreen touch gallery.
 *
 * @since 1.2.7
 *
 * @param array $vars Gallery configuration list.
 *
 * @return array
 */
function themify_theme_configure_gallery( $vars ) {
	$vars['contentImagesAreas'] = empty($vars['contentImagesAreas'])?'.gallery-wrapper':$vars['contentImagesAreas'].', .gallery-wrapper';
	return $vars;
}
add_filter( 'themify_gallery_plugins_args', 'themify_theme_configure_gallery' );

/**
 * Configure social share items.
 *
 * @since 1.4.0
 *
 * @param array $settings
 *
 * @return array
 */
function themify_theme_social_share_settings( $settings ) {
	$settings['share_html'] = apply_filters('themify_share_html', sprintf( '
			<div class="box">
				<a href="#" class="share">%s</a>
				<a href="#" class="share-icon"></a>
			</div>
			<div class="social-share clearfix">
				%s
			</div>
			<!-- .social-share -->',
			__( 'Share', 'themify' ),
			$settings['share_html']
		)
	);
	$settings['share_item'] = '.post-share';
	return $settings;
}
add_filter( 'themify_social_share_settings', 'themify_theme_social_share_settings' );

/**
 * Color animation effect for rows and header
 *
 * @since 1.4.2
 */
function themify_theme_do_animated_bg() {
    if(!themify_check( 'setting-color_animation_disabled',true )){
	// Set colors for color animation
	$color_key = 'setting-color_stop_';
	$colors = array();
	for ( $i = 1; $i <= 7; ++$i ) {
	    $color=themify_get_color( $color_key . $i,'',true);
	    if ( $color!=='' ) {
		$colors[] = $color;
	    }
	}
	// If there are no colors set in theme settings, use default colors
	if ( empty( $colors ) ) {
		$colors = array(
			'#33baab',
			'#e33b9e',
			'#4961d7',
			'#2ea85c',
			'#2bb8ed',
			'#dd5135',
		);
	}

	$selector = array(
		'.module_row.animated-bg'
	);
	/* when using transparent header or Background option is set, do not use the animated bg */
	if( ! ( themify_check( 'background_color' ) || themify_check( 'background_image' ) ||  'transparent' === themify_get( 'header_wrap',false ) ) ) {
		$selector[] = '#headerwrap';
	}

	$options = apply_filters( 'themify_animated_bg_params', array(
		'selector' => $selector,
		'speed' => ((int) themify_get( 'setting-color_animation_speed',5,true )) * 1000 ,
		'colors' => $colors
	) );

	$css_steps = '';
	foreach( $options['colors'] as $key => $val ) {
		$css_steps .= ( $key == 0 ? '0' : 100 / $key ) . '% { background-color: ' . $val . '; }' . "\n";
	}
	$selector = implode( ',', $options['selector'] );
	$speed = $options['speed'] * count( $colors ); // @note: in CSS the speed for the entire duration of the animation, so so we need to multiply the number of slides

	echo"
	<style id=\"tf_theme_bg_css\">
	@keyframes themifyAnimatedBG {
		{$css_steps}
	}
	{$selector} {
		animation:themifyAnimatedBG {$speed}ms infinite alternate
	}
	</style>
	";
    }
}
add_action( 'wp_head', 'themify_theme_do_animated_bg' );


/**
 * Get event repeat date of post
 *
 * @return string
 */
function themify_theme_get_repeat_date( $type, $number,$date, $time ) {
        $result = '';
        switch($type){
            case 'day':
                $result = sprintf(_n('Every day','Every %s days',$number,'themify'),$number);
            break;
            case 'week':
                $result = sprintf(_n('Every week on %2$s','Every %s weeks on %s',$number,'themify'),$number,  date_i18n('l',  strtotime($date)));
            break;
            case 'year':
                $result = sprintf(_n('Every year on %2$s','Every %s years on %s',$number,'themify'),$number,  date_i18n('M d',  strtotime($date)));
            break;
        }
        if($result && $time){
            $result.=_x( ' @ ', 'Connector between date and time (with spaces around itself) in event date and time.', 'themify' ).date_i18n( get_option( 'time_format' ), strtotime( $time ) ) ;
        }
        return apply_filters('themify_event_date_repeat',$result,$type, $number,$date, $time);
}

function themify_is_ajax_response(){
    static $is = NULL;
    if($is===null){
        $is = isset($_REQUEST['themify_ajax']) && !is_admin() && themify_is_ajax();
    }
    return  $is;
}
function themify_load_content_only(){
    return  !themify_is_ajax_response() || (themify_is_ajax_response() && !empty($_REQUEST['themify_load']));
}
function themify_ajax_response(){
    if(themify_is_ajax_response()){
        
        add_filter('themify_builder_is_frontend_editor','__return_false');
        add_action( 'themify_footer_data', 'themify_render_ajax_files',100);
       
    }
}

function themify_render_ajax_files(){
    global $wp_scripts,$wp_styles; 
    $response_js = array('queue'=>$wp_scripts->queue,'registered'=>$wp_scripts->registered);
    echo '<script>themify_js_files='.  json_encode($response_js).'</script>';
}

function themify_preprocess_comment(){
    if(!empty($_POST['data'])){
        $response = array();
        $comment = wp_handle_comment_submission( wp_unslash( $_POST['data'] ) );
        if ( is_wp_error( $comment ) ) {
            $error = $comment->get_error_message();
            $response['error'] = 1;
            $known_errors = array(
                'require_name_email'=>'author,email',
                'require_valid_email'=>'email',
                'comment_author_column_length'=>'author',
                'comment_author_email_column_length'=>'email',
                'comment_author_url_column_length'=>'url',
                'require_valid_comment'=>'comment',
                'comment_content_column_length'=>'comment'
            );
            $key = $comment->get_error_code();
            $response['data']['val'] = $comment->get_error_message();
            if(isset($known_errors[$key])){
                $response['data']['id']=$known_errors[$key];
            }
        }
        else{
            
            $user = wp_get_current_user();
            do_action( 'set_comment_cookies', $comment, $user );
            $response['success'] = 1;
            $comment_post_ID = $_POST['data']['comment_post_ID'];
            global $post, $product;
            wp_update_comment_count($comment_post_ID);
            $post = get_post( $comment_post_ID );
            query_posts(array('p'=>$post->ID,'post_type'=>$post->post_type));
            if(class_exists('WC_Product')){
                $product = new WC_Product($post);
            }
            ob_start();
            comments_template();
            $response['data'] = ob_get_contents();
            ob_end_clean();
        }
        echo wp_json_encode($response);
    }
    wp_die();
}
add_action('wp_ajax_themify_comment', 'themify_preprocess_comment');

add_action('wp_ajax_nopriv_themify_comment', 'themify_preprocess_comment');
function themify_theme_fixed_header() {
        static $is = NULL;
        if($is===null){
            $header = themify_get_header_design_value();
            if ( in_array( $header, array( 'header-leftpane', 'header-rightpane', 'header-minbar', 'none' ),true ) ) {
                    $is = '';
                    return $is;
            }
	    $fixed_header_field = themify_get( 'fixed_header' );
	    if ( 'yes' == $fixed_header_field ) {
		$is = 'fixed-header-enabled';
		return $is;
	    } elseif ( 'no' == $fixed_header_field ) {
		    $is = '';
		    return $is;
	    }
            $is = themify_check( 'setting-fixed_header_disabled',true ) ? '' : 'fixed-header-enabled';
        }
        return $is;
}

function themify_theme_sticky_logo(){
     if(themify_theme_fixed_header()){
        global $themify_customizer;
        $logo = json_decode($themify_customizer->get_cached_mod('sticky_header_imageselect'));
        return isset($logo->src) && '' != $logo->src?$logo:false;
     }
     else{
         return false;
     }
}

function themif_is_event_expired( $event_end ) {
	$event_end = strtotime( $event_end );

	return ! empty( $event_end ) && is_numeric( $event_end ) ? time() > $event_end : false;
}

function themify_theme_loops_wrapper_class($class,$post_type,$layout,$type,$moduleArgs=array(),$slug=false) {
	if($post_type!=='' && $post_type!=='post'){
		if($post_type==='gallery'){
		$class[]='no-gutter';
		}
		if(($layout==='grid2' || $layout==='slider' || $layout==='grid3' || $layout==='grid4' || $layout==='grid5' || $layout==='grid6') && ($post_type==='video' || $post_type==='album' || $post_type==='gallery' || $post_type==='event')){
		if($post_type==='event'){
			if($layout==='slider'){
			$class[]='tf_grid_content';
			}
		}
		else{
			$class[]='tf_grid_content';
		}
		}
		Themify_Shortcode_Base::loadStyles($post_type,$layout );
	}
	return $class;
}
add_filter('themify_loops_wrapper_class','themify_theme_loops_wrapper_class',10,6);
